<?php
/**
 * Troll storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required parameters:
 * ====================
 *   'database'  --  The name of the database.
 *   'hostspec'  --  The hostname of the database server.
 *   'password'  --  The password associated with 'username'.
 *   'phptype'   --  The database type (ie. 'pgsql', 'mysql, etc.).
 *   'protocol'  --  The communication protocol ('tcp', 'unix', etc.).
 *   'username'  --  The username with which to connect to the database.
 *
 * Copyright 2003-2005 Chris Bowlby <excalibur@hub.org>
 * Copyright 2003-2005 Max Kalika <max@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chris Bowlby <excalibur@hub.org>
 * @author  Max Kalika <max@horde.org>
 * @package Troll
 */
class Troll_Driver_sql extends Troll_Driver {

    /**
     * Handle for the current database connection.
     *
     * @var object DB $_db
     */
    var $_db;

    /**
     * Boolean indicating whether or not we're connected to the SQL server.
     *
     * @var boolean $_connected
     */
    var $_connected = false;

    /**
     * Constructs a new SQL storage object.
     *
     * @access public
     *
     * @param optional array $params  A hash containing connection
     *                                parameters.
     */
    function Troll_Driver_sql($user, $params = array())
    {
        $this->_user = $user;
        $this->_params = array_merge($GLOBALS['conf']['sql'], $params);
    }

    /**
     * Retrieve an article list set from the storage backend.
     *
     * @access public
     *
     * @param string $group  Newsgroup to get article list for.
     *
     * @return mixed  Array of option-value pairs or a PEAR_Error object
     *                on failure.
     */
    function retrieve($group)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE owner_id = %s AND newsgroup = %s',
                         $this->_params['table'],
                         $this->_db->quote($this->_user),
                         $this->_db->quote($group));

        /* Execute the query. */
        $result = $this->_db->query($query);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
        $result->free();

        if (is_a($row, 'PEAR_Error')) {
            return $row;
        }

        return array($row['newsgroup'] => $row['articles']);
    }

    /**
     * Store an article list to the storage backend.
     *
     * @access public
     *
     * @param string $group    Newsgroup for which to store article list.
     * @param array $articles  List of read articles to store.
     *
     * @return mixed  True on success or a PEAR_Error object on failure.
     */
    function store($group, $articles)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        if (is_null($articles)){
            $this->_db->query(sprintf('DELETE FROM %s WHERE owner_id = %s '.
                                      'AND newsgroup = %s;',
                                      $this->_params['table'],
                                      $this->_db->quote($this->_user),
                                      $this->_db->quote($group)));
        } else {
            $query = sprintf('SELECT 1 FROM %s WHERE owner_id = %s ' .
                             'AND newsgroup = %s',
                             $this->_params['table'],
                             $this->_db->quote($this->_user),
                             $this->_db->quote($group));

            $result = $this->_db->getOne($query);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            if (is_null($result)) {
                $query = sprintf('INSERT INTO %s (owner_id, newsgroup, articles) ' .
                                 'VALUES (%s, %s, %s)',
                                 $this->_params['table'],
                                 $this->_db->quote($this->_user),
                                 $this->_db->quote($group),
                                 $this->_db->quote($articles));
            } else {
                $query = sprintf('UPDATE %s SET articles = %s WHERE owner_id = %s ' .
                                 'AND newsgroup = %s',
                                 $this->_params['table'],
                                 $this->_db->quote($articles),
                                 $this->_db->quote($this->_user),
                                 $this->_db->quote($group));
            }
        }

        $result = $this->_db->query($query);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return true;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @access private
     *
     * @return mixed  True on success or a PEAR_Error object on failure.
     */
    function _connect()
    {
        if ($this->_connected) {
            return true;
        }

        Horde::assertDriverConfig($this->_params, 'sql',
                                  array('phptype', 'hostspec', 'username', 'database'),
                                  'Troll backend', 'backends.php', '$backends');
        if (!isset($this->_params['table'])) {
            $this->_params['table'] = 'troll_articles';
        }

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_db = &DB::connect($this->_params, array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_db, 'PEAR_Error')) {
            Horde::fatal($this->_db, __FILE__, __LINE__);
        }

        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

        $this->_connected = true;

        return true;
    }

}
