<?php
/**
 * The Troll_NNTP:: class facilitates connections to the NNTP server
 * via the c-client PHP extension.
 *
 * $Horde: troll/lib/NNTP.php,v 1.5 2005/01/03 14:35:35 jan Exp $
 *
 * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 2002-2005 Jon Parise <chuck@horde.org>
 * Copyright 2004-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.

 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@horde.org>
 * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
 * @version $Revision: 1.5 $
 * @package Troll
 */
class Troll_NNTP {

    /**
     * The username for the server.
     *
     * @var string $_user
     */
    var $_user;

    /**
     * The password for the mail server.
     *
     * @var string $_pass
     */
    var $_pass;

    /**
     * The currently open group.
     *
     * @var string $_openGroup
     */
    var $_openGroup = null;

    /**
     * The IMAP flags set in the currently open mailbox.
     *
     * @var integer $_flags
     */
    var $_flags = null;

    /**
     * Attempts to return a reference to a concrete Troll_NNTP instance.
     * It will only create a new instance if no Troll_NNTP instance currently
     * exists.
     *
     * This method must be invoked as:
     *   $nntp = &Troll_NNTP::singleton();
     *
     * @access public
     *
     * @param optional array $params  Any parameters needed for the
     *                                constructor.
     *
     * @return object Troll_NNTP  The concrete Troll_NNTP reference, or
     *                            false on error.
     */
    function &singleton($params = array())
    {
        static $instance;

        if (!isset($instance)) {
            $instance = new Troll_NNTP($params);
        }

        return $instance;
    }

    /**
     * Constructor.
     *
     * @access public
     *
     * @param optional array $params  Any additional parameters needed.
     */
    function Troll_NNTP($params = array())
    {
        global $prefs;

        $this->_user = $prefs->getValue('auth_user');
        $this->_pass = $prefs->getValue('auth_pass');
    }

    /**
     * Open an NNTP stream.
     *
     * @access public
     *
     * @param optional string $group   A newsgroup to open.
     * @param optional integer $flags  Any flags that need to be passed to
     *                                 imap_open().
     *
     * @return resource  The return from the imap_open() call.
     */
    function &openNNTPStream($group = null, $flags = 0)
    {
        $flags = OP_ANONYMOUS;
        if (!is_null($group)) {
            $group = imap_utf7_encode($group);
        } else {
            $flags |= OP_HALFOPEN;
        }

        $res = @imap_open(Troll::serverString() . $group, $this->_user, $this->_pass, $flags);

        if (!$res) {
            Horde::fatal(PEAR::raiseError(imap_last_error()), __FILE__, __LINE__);
        } else {
            return $res;
        }
    }

    /**
     * Change the currently active IMP NNTP stream to a new group
     * (if necessary).
     *
     * @access public
     *
     * @param string $group            The new group.
     * @param optional integer $flags  Any flags that need to be passed to
     *                                 imap_reopen().
     *
     * @return boolean  True on success, false on error.
     */
    function changeGroup($group, $flags = null)
    {
        /* Open a connection if none exists. */
        if (empty($_SESSION['troll']['stream'])) {
            if (($_SESSION['troll']['stream'] = $this->openNNTPStream($group, $flags))) {
                $this->_openGroup = $group;
                $this->_flags = $flags;
                return true;
            } else {
                return false;
            }
        }

        /* Only reopen group if we need to - either we are changing
         * groups or the flags for the current group has changed. */
        if (($this->_openGroup != $group) || ($this->_flags != $flags)) {
            $result = @imap_reopen($_SESSION['troll']['stream'], $this->_serverString . $group, $flags);
            if ($result) {
                $this->_openGroup = $group;
                $this->_flags = $flags;
                return true;
            } else {
                return false;
            }
        }

        return true;
    }

    /**
     * Reopen a NNTP stream.
     *
     * @access public
     *
     * @return boolean  See Auth_imp::authenticate().
     */
    function reopenIMAPStream($expunge = false)
    {
        $flags = 0;
        if ($expunge) {
            $flags = CL_EXPUNGE;
        }

        @imap_close($_SESSION['imp']['stream'], $flags);
        unset($_SESSION['imp']['stream']);

        return IMP::checkAuthentication(null, true);
    }

    /**
     * Open a stream using the Net_NNTP PEAR library.
     *
     * @access public
     *
     * @return object Net_NNTP  The Net_NNTP object or PEAR_Error on error.
     */
    function &getNetNNTPOb()
    {
        require_once 'Net/NNTP.php';
        $nntp = &new Net_NNTP();
        $nntp->connectAuthenticated($this->_user, $this->_pass, $GLOBALS['conf']['news']['server']);

        return $nntp;
    }

}
